@@ -1,140 +1,141 @@
-/*  $Revision: 1.11 $
+/*  $Id: ctlinnd.c 7625 2007-08-02 14:21:58Z iulius $
 **
 **  Send control messages to the InterNetNews daemon.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
 #include <ctype.h>
 #include <errno.h>
 #include <sys/stat.h>
-#include "libinn.h"
-#include "inndcomm.h"
-#include "paths.h"
-#include "macros.h"
-#include <syslog.h>  
+
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "inn/inndcomm.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 
 
 /*
 **  Datatype for an entry in the command table.
 */
 typedef struct _COMMAND {
-    STRING	Command;
-    STRING	Text;
+    const char *Command;
+    const char *Text;
     int		argc;
     char	Letter;
-    BOOL	Glue;
+    bool	Glue;
 } COMMAND;
 
 
-STATIC COMMAND	Commands[] = {
-    {	"addhist",	"id arr exp post path...\tAdd history line",
-	5,	SC_ADDHIST,	TRUE	},
+static COMMAND	Commands[] = {
+    {	"addhist",	"id arr exp post token...\tAdd history line",
+	5,	SC_ADDHIST,	true	},
     {	"allow",	"reason...\t\t\tAllow remote connections",
-	1,	SC_ALLOW,	TRUE	},
+	1,	SC_ALLOW,	true	},
     {	"begin",	"site\t\t\tStart newly-added site",
-	1,	SC_BEGIN,	FALSE	},
+	1,	SC_BEGIN,	false	},
     {	"cancel",	"id\t\t\tCancel message locally",
-	1,	SC_CANCEL,	FALSE	},
+	1,	SC_CANCEL,	false	},
     {	"changegroup",	"group rest\tChange mode of group",
-	2,	SC_CHANGEGROUP,	FALSE	},
+	2,	SC_CHANGEGROUP,	false	},
     {	"checkfile",	"\t\t\tCheck syntax of newsfeeds file",
-	0,	SC_CHECKFILE,	FALSE	},
+	0,	SC_CHECKFILE,	false	},
     {	"drop",		"site\t\t\tStop feeding site",
-	1,	SC_DROP,	FALSE		},
+	1,	SC_DROP,	false		},
     {	"feedinfo",		"site\t\t\tPrint state of feed to site*",
-	1,	SC_FEEDINFO,	FALSE		},
-#if defined(DO_TCL)
-    {	"tcl",			"flag\t\t\tEnable or disable Tcl filtering",
-	1,	SC_FILTER,	FALSE		},
-#endif /* defined(DO_TCL) */
+	1,	SC_FEEDINFO,	false		},
     {	"flush",	"site\t\t\tFlush feed for site*",
-	1,	SC_FLUSH,	FALSE	},
+	1,	SC_FLUSH,	false	},
     {	"flushlogs",	"\t\t\tFlush log files",
-	0,	SC_FLUSHLOGS,	FALSE	},
+	0,	SC_FLUSHLOGS,	false	},
     {	"go",		"reason...\t\t\tRestart after pause or throttle",
-	1,	SC_GO,		TRUE	},
+	1,	SC_GO,		true	},
     {	"hangup",	"channel\t\tHangup specified incoming channel",
-	1,	SC_HANGUP,	FALSE	},
+	1,	SC_HANGUP,	false	},
     {	"logmode",		"\t\t\t\tSend server mode to syslog",
-	0,	SC_LOGMODE,	FALSE		},
+	0,	SC_LOGMODE,	false		},
     {	"mode",		"\t\t\t\tPrint operating mode",
-	0,	SC_MODE,	FALSE		},
+	0,	SC_MODE,	false		},
     {	"name",		"nnn\t\t\tPrint name of specified channel*",
-	1,	SC_NAME,	FALSE		},
+	1,	SC_NAME,	false		},
     {	"newgroup",	"group rest creator\tCreate new group",
-	3,	SC_NEWGROUP,	FALSE	},
+	3,	SC_NEWGROUP,	false	},
     {	"param",	"letter value\t\tChange command-line parameters",
-	2,	SC_PARAM,	FALSE	},
+	2,	SC_PARAM,	false	},
     {	"pause",	"reason...\t\tShort-term pause in accepting articles",
-	1,	SC_PAUSE,	TRUE	},
-#if defined(DO_PERL)
+	1,	SC_PAUSE,	true	},
+#ifdef DO_PERL
     {	"perl",			"flag\t\t\tEnable or disable Perl filtering",
-	1,	SC_PERL,	FALSE	},
-#endif /* defined(DO_PERL) */
+	1,	SC_PERL,	false	},
+#endif
+#ifdef DO_PYTHON
+    {	"python",		"flag\t\t\tEnable or disable Python filtering",
+	1,	SC_PYTHON,	false	},
+#endif
     {	"readers",	"flag text...\t\tEnable or disable newsreading",
-	2,	SC_READERS,	TRUE	},
+	2,	SC_READERS,	true	},
     {	"reject",	"reason...\t\t\tReject remote connections",
-	1,	SC_REJECT,	TRUE	},
+	1,	SC_REJECT,	true	},
     {	"reload",	"what reason...\t\tRe-read config files*",
-	2,	SC_RELOAD,	TRUE	},
+	2,	SC_RELOAD,	true	},
     {	"renumber",	"group\t\tRenumber the active file*",
-	1,	SC_RENUMBER,	FALSE	},
+	1,	SC_RENUMBER,	false	},
     {	"reserve",	"reason...\t\tReserve the next pause or throttle",
-	1,	SC_RESERVE,	TRUE	},
+	1,	SC_RESERVE,	true	},
     {	"rmgroup",	"group\t\t\tRemove named group",
-	1,	SC_RMGROUP,	FALSE	},
+	1,	SC_RMGROUP,	false	},
     {	"send",		"feed text...\t\tSend text to exploder feed",
-	2,	SC_SEND,	TRUE	},
+	2,	SC_SEND,	true	},
     {	"shutdown",	"reason...\t\tShut down server",
-	1,	SC_SHUTDOWN,	TRUE	},
+	1,	SC_SHUTDOWN,	true	},
+    {	"stathist",	"filename|off\t\tLog into filename some history stats",
+	1,	SC_STATHIST,	false	},
     {	"status",	"interval|off\t\tTurn innd status generation on or off",
-	1,	SC_STATUS,	FALSE	},
+	1,	SC_STATUS,	false	},
     {	"kill",	"signal site\t\tSend signal to site's process",
-	2,	SC_SIGNAL,	FALSE	},
+	2,	SC_SIGNAL,	false	},
     {	"throttle",	"reason...\t\tStop accepting articles",
-	1,	SC_THROTTLE,	TRUE	},
+	1,	SC_THROTTLE,	true	},
     {   "timer",	"interval|off\t\tTurn performance monitoring on or off",
-	1,	SC_TIMER,	FALSE	},
+	1,	SC_TIMER,	false	},
     {	"trace",	"innd|#|nnrpd flag\tTurn tracing on or off",
-	2,	SC_TRACE,	FALSE	},
+	2,	SC_TRACE,	false	},
     {	"xabort",	"text...\t\tAbort the server",
-	1,	SC_XABORT,	TRUE	},
+	1,	SC_XABORT,	true	},
     { "lowmark",	"filename\t\tReset active file low article marks",
-	1,	SC_LOWMARK,	FALSE	},
+	1,	SC_LOWMARK,	false	},
     { "renumberlow",	"filename\t\tReset active file low article marks",
-	1,	SC_LOWMARK,	FALSE	},
+	1,	SC_LOWMARK,	false	},
     {	"xexec",	"path\t\t\tExec new server",
-	1,	SC_XEXEC,	FALSE	}
+	1,	SC_XEXEC,	false	}
 };
 
-
+
 
 /*
 **  Print a help summary.
 */
-STATIC NORETURN
-Help(p)
-    char		*p;
+static void
+Help(char *p)
 {
-    register COMMAND	*cp;
+    COMMAND	*cp;
 
     if (p == NULL) {
-	(void)printf("Command summary:\n");
-	for (cp = Commands; cp < ENDOF(Commands); cp++)
-	    (void)printf("  %s %s\n", cp->Command, cp->Text);
-	(void)printf("*   Empty string means all sites/groups/etc.\n");
-	(void)printf("... All trailing words are glued together.\n");
+	printf("Command summary:\n");
+	for (cp = Commands; cp < ARRAY_END(Commands); cp++)
+	    printf("  %s %s\n", cp->Command, cp->Text);
+	printf("*   Empty string means all sites/groups/etc.\n");
+	printf("... All trailing words are glued together.\n");
 	exit(0);
     }
-    for (cp = Commands; cp < ENDOF(Commands); cp++)
-	if (EQ(p, cp->Command)) {
-	    (void)printf("Command usage:\n");
-	    (void)printf("  %s %s\n", cp->Command, cp->Text);
+    for (cp = Commands; cp < ARRAY_END(Commands); cp++)
+	if (strcmp(p, cp->Command) == 0) {
+	    printf("Command usage:\n");
+	    printf("  %s %s\n", cp->Command, cp->Text);
 	    exit(0);
 	}
-    (void)printf("No such command.\n");
+    printf("No such command.\n");
     exit(0);
 }
 
@@ -142,12 +143,11 @@ Help(p)
 /*
 **  Print a command-usage message and exit.
 */
-STATIC NORETURN
-WrongArgs(cp)
-    COMMAND	*cp;
+static void
+WrongArgs(COMMAND *cp)
 {
-    (void)printf("Wrong number of arguments -- usage:\n");
-    (void)printf("  %s %s\n", cp->Command, cp->Text);
+    printf("Wrong number of arguments -- usage:\n");
+    printf("  %s %s\n", cp->Command, cp->Text);
     exit(1);
 }
 
@@ -155,16 +155,14 @@ WrongArgs(cp)
 /*
 **  Print an error message and exit.
 */
-STATIC NORETURN
-Failed(p)
-    char	*p;
+static void
+Failed(const char *p)
 {
     if (ICCfailure)
-	(void)fprintf(stderr, "Can't %s (%s failure) %s.\n",
-		p, ICCfailure, strerror(errno));
+        syswarn("cannot %s (%s failure)", p, ICCfailure);
     else
-	(void)fprintf(stderr, "Can't %s, %s.\n", p, strerror(errno));
-    (void)ICCclose();
+        syswarn("cannot %s", p);
+    ICCclose();
     exit(1);
 }
 
@@ -172,11 +170,10 @@ Failed(p)
 /*
 **  Print an error reporting incorrect usage.
 */
-STATIC NORETURN
-Usage(what)
-    char	*what;
+static void
+Usage(const char *what)
 {
-    (void)fprintf(stderr, "Usage error (%s) -- try -h for help.\n", what);
+    fprintf(stderr, "Usage error (%s) -- try -h for help.\n", what);
     exit(1);
 }
 
@@ -188,8 +185,8 @@ int main(int ac, char *av[])
     COMMAND	        *cp;
     char	        *p;
     int	                i;
-    BOOL		Silent;
-    BOOL		NeedHelp;
+    bool		Silent;
+    bool		NeedHelp;
     char		*reply;
     char		*new;
     int			length;
@@ -197,13 +194,14 @@ int main(int ac, char *av[])
     struct stat		Sb;
     char		buff[SMBUF];
 
-    /* First thing, set up logging and our identity. */
-    openlog("ctlinnd", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);        
+    /* First thing, set up our identity. */
+    message_program_name = "ctlinnd";
 
     /* Set defaults. */
-    if (ReadInnConf() < 0) exit(1);
-    Silent = FALSE;
-    NeedHelp = FALSE;
+    if (!innconf_read(NULL))
+        exit(1);
+    Silent = false;
+    NeedHelp = false;
     ICCsettimeout(CTLINND_TIMEOUT);
 
     /* Parse JCL. */
@@ -213,10 +211,10 @@ int main(int ac, char *av[])
 	    Usage("bad flags");
 	    /* NOTREACHED */
 	case 'h':		/* Get help			*/
-	    NeedHelp = TRUE;
+	    NeedHelp = true;
 	    break;
 	case 's':		/* Silent -- no output		*/
-	    Silent = TRUE;
+	    Silent = true;
 	    break;
 	case 't':		/* Time to wait for reply	*/
 	    ICCsettimeout(atoi(optarg));
@@ -230,12 +228,12 @@ int main(int ac, char *av[])
 	Usage("missing command");
 
     /* Look up the command word and move to the arguments. */
-    if (EQ(av[0], "help"))
+    if (strcmp(av[0], "help") == 0)
 	Help(av[1]);
-    for (cp = Commands; cp < ENDOF(Commands); cp++)
-	if (EQ(av[0], cp->Command))
+    for (cp = Commands; cp < ARRAY_END(Commands); cp++)
+	if (strcmp(av[0], cp->Command) == 0)
 	    break;
-    if (cp == ENDOF(Commands))
+    if (cp == ARRAY_END(Commands))
 	Usage("unknown command");
     ac--;
     av++;
@@ -264,6 +262,28 @@ int main(int ac, char *av[])
 	case 3:
 	    break;
 	}
+	ac = 3;
+    }
+    else if (ac > cp->argc && cp->Glue) {
+	/* Glue any extra words together. */
+	for (length = 0, i = cp->argc - 1; (p = av[i++]) != NULL; )
+	    length += strlen(p) + 1;
+        new = xmalloc(length);
+        *new = '\0';
+	for (i = cp->argc - 1; av[i]; i++) {
+	    if (i >= cp->argc)
+                strlcat(new, " ", length);
+            strlcat(new, av[i], length);
+	}
+	av[cp->argc - 1] = new;
+	av[cp->argc] = NULL;
+    }
+    else if (ac != cp->argc)
+	/* All other commands must have the right number of arguments. */
+	WrongArgs(cp);
+
+    /* For newgroup and changegroup, make sure the mode is valid. */
+    if (cp->Letter == SC_NEWGROUP || cp->Letter == SC_CHANGEGROUP) {
 	switch (av[1][0]) {
 	default:
 	    Usage("Bad group mode");
@@ -276,42 +296,24 @@ int main(int ac, char *av[])
 	case NF_FLAG_IGNORE:
 	    break;
 	}
-	ac = 3;
     }
-    else if (ac > cp->argc && cp->Glue) {
-	/* Glue any extra words together. */
-	for (length = 0, i = cp->argc - 1; (p = av[i++]) != NULL; )
-	    length += strlen(p) + 1;
-	for (new = p = NEW(char, length), i = cp->argc - 1; av[i]; i++) {
-	    if (i >= cp->argc)
-		*p++ = ' ';
-	    p += strlen(strcpy(p, av[i]));
-	}
-	av[cp->argc - 1] = new;
-	av[cp->argc] = NULL;
-    }
-    else if (ac != cp->argc)
-	/* All other commands must have the right number of arguments. */
-	WrongArgs(cp);
 
     /* Make sure there are no separators in the parameters. */
     for (i = 0; (p = av[i++]) != NULL; )
-	if (strchr(p, SC_SEP) != NULL) {
-	    (void)fprintf(stderr, "Illegal character '\\%03o' in \"%s\"\n",
-		    SC_SEP, p);
-	    exit(1);
-	}
+	if (strchr(p, SC_SEP) != NULL)
+            die("illegal character \\%03o in %s", SC_SEP, p);
 
     /* Do the real work. */
     if (ICCopen() < 0)
 	Failed("setup communication");
-    i = ICCcommand(cp->Letter, av, &reply);
+    i = ICCcommand(cp->Letter, (const char **) av, &reply);
     if (i < 0) {
 	i = errno;
-	p = cpcatpath(innconf->pathrun, _PATH_SERVERPID);
+	p = concatpath(innconf->pathrun, INN_PATH_SERVERPID);
 	if (stat(p, &Sb) < 0)
-	    (void)fprintf(stderr, "No innd.pid file; did server die?\n");
-	(void)sprintf(buff, "send \"%s\" command", cp->Command);
+            warn("no innd.pid file; did server die?");
+        free(p);
+	snprintf(buff, sizeof(buff), "send \"%s\" command", cp->Command);
 	errno = i;
 	Failed(buff);
     }
@@ -323,9 +325,9 @@ int main(int ac, char *av[])
 	while (*p && ISWHITE(*p))
 	    p++;
 	if (i != 0)
-	    (void)fprintf(stderr, "%s\n", p);
+            warn("%s", p);
 	else if (!Silent)
-	    (void)printf("%s\n", p);
+	    printf("%s\n", p);
     }
 
     if (ICCclose() < 0)
