@@ -1,7 +1,7 @@
-/*  $Id: innd.h 7858 2008-06-05 18:51:20Z iulius $
+/*  $Id: innd.h 8468 2009-05-16 12:28:24Z iulius $
 **
-**  Many of the data types used here have abbreviations, such as CT
-**  for CHANNELTYPE.  Here are a list of the conventions and meanings:
+**  Many of the data types used here have abbreviations, such as CT for a
+**  channel type.  Here are a list of the conventions and meanings:
 **
 **    ART   A news article
 **    CHAN  An I/O channel
@@ -41,16 +41,10 @@
 #include "inn/history.h"
 #include "inn/messages.h"
 #include "inn/timer.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 #include "nntp.h"
-#include "paths.h"
-#include "storage.h"
-
-/* TCL defines EXTERN, so undef it after inclusion since we use it. */
-#if DO_TCL
-# include <tcl.h>
-# undef EXTERN
-#endif
+#include "inn/paths.h"
+#include "inn/storage.h"
 
 BEGIN_DECLS
 
@@ -72,12 +66,15 @@ typedef short	SITEIDX;
 
 
 /*
-**  Server's operating mode.
+**  Server's operating mode.  Note that OMshutdown is only used internally
+**  while shutting down in CCshutdown or CCxexec and does not need to be
+**  handled outside of the Perl and Python filtering code.
 */
 typedef enum _OPERATINGMODE {
   OMrunning,
   OMpaused,
-  OMthrottled
+  OMthrottled,
+  OMshutdown
 } OPERATINGMODE;
 
 
@@ -205,29 +202,33 @@ typedef struct _HDRCONTENT {
 #define HDR__IN_REPLY_TO	63
 #define HDR__INJECTION_DATE	64
 #define HDR__NNTP_POSTING_DATE	65
+#define HDR__X_USER_ID          66
+#define HDR__X_AUTH_SENDER      67
+#define HDR__X_ORIGINAL_NNTP_POSTING_HOST       68
+#define HDR__ORIGINAL_SENDER    69
+#define HDR__NNTP_POSTING_PATH  70
 
-#define MAX_ARTHEADER           66
+#define MAX_ARTHEADER           71
 
 /*
 **  Miscellaneous data we want to keep on an article.  All the fields
 **  are not always valid.
 */
 typedef struct _ARTDATA {
-  int		  Body;			/* where body begins in article
+  size_t	  Body;			/* where body begins in article
 					   it indicates offset from bp->Data */
-  char	      *   Poster;		/* Sender otherwise From in article */
-  char	      *   Replyto;		/* Reply-To otherwise From in article */
   time_t	  Posted;		/* when article posted */
   time_t	  Arrived;		/* when article arrived */
   time_t	  Expires;		/* when article should be expired */
   int		  Lines;		/* number of body lines */
   int		  HeaderLines;		/* number of header lines */
+  char            LinesBuffer[SMBUF];   /* Generated Lines: header. */
   long		  BytesValue;		/* size of stored article, "\r\n" is
-					   counted as 1 byte */
-  char		  Bytes[16];		/* generated Bytes header */
-  int		  BytesLength;		/* generated Bytes header length */
-  char	      *   BytesHeader;		/* where Bytes header begins in
-					   received article */
+					   counted as 2 bytes */
+  char            Bytes[SMBUF];         /* Generated Bytes: header. */
+  int             BytesLength;          /* Generated Bytes: header length. */
+  char        *   BytesHeader;          /* Where Bytes: header begins in
+                                           received article. */
   char		  TokenText[(sizeof(TOKEN) * 2) + 3];
 					/* token of stored article */
   LISTBUFFER	  Newsgroups;		/* newsgroup list */
@@ -250,16 +251,11 @@ typedef struct _ARTDATA {
   struct buffer	  Overview;		/* buffer for overview data */
   int		  CRwithoutLF;		/* counter for '\r' without '\n' */
   int		  LFwithoutCR;		/* counter for '\n' without '\r' */
-  long		  CurHeader;		/* where current header starts.
+  int             DotStuffedLines;      /* counter for lines beginning with '.' */
+  size_t	  CurHeader;		/* where current header starts.
 					   this is used for folded header
 					   it indicates offset from bp->Data */
-  bool		  NullHeader;		/* contains NULL in current header    */
-  long		  LastTerminator;	/* where last '.' exists.  only set if
-					   it exists at the begining of line
-					   it indicates offset from bp->Data */
-  long		  LastCR;		/* where last CR exists
-					   it indicates offset from bp->Data */
-  long		  LastCRLF;		/* where last CRLF exists.
+  size_t	  LastCRLF;		/* where last CRLF exists.
 					   indicates where last LF exists
 					   it indicates offset from bp->Data */
   HDRCONTENT	  HdrContent[MAX_ARTHEADER];
@@ -271,47 +267,45 @@ typedef struct _ARTDATA {
                                            Pathcluster */
 } ARTDATA;
 
-/*
-**  Set of channel types.
-*/
-typedef enum _CHANNELTYPE {
-  CTany,
-  CTfree,
-  CTremconn,
-  CTreject,
-  CTnntp,
-  CTlocalconn,
-  CTcontrol,
-  CTfile,
-  CTexploder,
-  CTprocess
-} CHANNELTYPE;
 
+/* Set of channel types. */
+enum channel_type {
+    CTany,
+    CTfree,
+    CTremconn,
+    CTreject,
+    CTnntp,
+    CTlocalconn,
+    CTcontrol,
+    CTfile,
+    CTexploder,
+    CTprocess
+};
+
+/* The state a channel is in.  Interpretation of this depends on the channel's
+   type.  Used mostly by CTnntp channels. */
+enum channel_state {
+    CSerror,
+    CSwaiting,
+    CSgetcmd,
+    CSgetauth,
+    CSwritegoodbye,
+    CSwriting,
+    CSpaused,
+    CSgetheader,
+    CSgetbody,
+    CSgotarticle,
+    CSgotlargearticle,
+    CSnoarticle,
+    CSeatarticle,
+    CSeatcommand,
+    CSgetxbatch,
+    CScancel
+};
 
-/*
-**  The state a channel is in.  Interpretation of this depends on the
-**  channel's type.  Used mostly by CTnntp channels.
-*/
-typedef enum _CHANNELSTATE {
-  CSerror,
-  CSwaiting,
-  CSgetcmd,
-  CSgetauth,
-  CSwritegoodbye,
-  CSwriting,
-  CSpaused,
-  CSgetheader,
-  CSgetbody,
-  CSgotarticle,
-  CSgotlargearticle,
-  CSnoarticle,
-  CSeatarticle,
-  CSeatcommand,
-  CSgetxbatch,
-  CScancel
-} CHANNELSTATE;
 
 #define SAVE_AMT	10	/* used for eating article/command */
+#define PRECOMMITCACHESIZE 128
 
 /*
 **  I/O channel, the heart of the program.  A channel has input and output
@@ -319,19 +313,20 @@ typedef enum _CHANNELSTATE {
 **  all the output was been written.  Many callback functions take a
 **  pointer to a channel, so set up a typedef for that.
 */
-#define PRECOMMITCACHESIZE 128
 struct _CHANNEL;
-typedef void (*innd_callback_t)(struct _CHANNEL *);
+typedef void (*innd_callback_func)(struct _CHANNEL *);
 
 typedef struct _CHANNEL {
-  CHANNELTYPE	       Type;
-  CHANNELSTATE	       State;
+  enum channel_type    Type;
+  enum channel_state   State;
   int		       fd;
   bool		       Skip;
+  bool                 Ignore;
   bool		       Streaming;
   bool		       NoResendId;
   bool		       privileged;
   bool		       Nolist;
+  bool                 CanAuthenticate; /* Can use AUTHINFO? */
   unsigned long	       Duplicate;
   unsigned long	       Unwanted_s;
   unsigned long	       Unwanted_f;
@@ -341,6 +336,7 @@ typedef struct _CHANNEL {
   unsigned long	       Unwanted_o;
   float		       Size;
   float		       DuplicateSize;
+  float		       RejectSize;
   unsigned long	       Check;
   unsigned long	       Check_send;
   unsigned long	       Check_deferred;
@@ -365,11 +361,11 @@ typedef struct _CHANNEL {
   time_t	       LastActive;
   time_t	       NextLog;
   struct sockaddr_storage Address;
-  innd_callback_t      Reader;
-  innd_callback_t      WriteDone;
+  innd_callback_func   Reader;
+  innd_callback_func   WriteDone;
   time_t	       Waketime;
   time_t	       Started;
-  innd_callback_t      Waker;
+  innd_callback_func   Waker;
   void		    *  Argument;
   void		    *  Event;
   struct buffer	       In;
@@ -387,19 +383,25 @@ typedef struct _CHANNEL {
   int		       HoldTime;
   time_t	       ArtBeg;
   int		       ArtMax;
-  long		       Start;		/* where current cmd/article starts
+  size_t	       Start;		/* where current cmd/article starts
 					   it indicates offset from bp->Data */
-  long		       Next;		/* next pointer to read
+  size_t	       Next;		/* next pointer to read
 					   it indicates offset from bp->Data */
   char		       Error[SMBUF];	/* error buffer */
   ARTDATA	       Data;		/* used for processing article */
-  struct _CHANNEL      *nextcp;		/* linked list for each incoming site */
+  char                 Name[SMBUF];     /* storage for CHANname */
 } CHANNEL;
 
 #define	DEFAULTNGBOXSIZE	64
 
 /*
-**  A newsgroup has a name in different formats, and a high-water count,
+**  Different types of rejected articles.
+*/
+typedef enum {REJECT_DUPLICATE, REJECT_SITE, REJECT_FILTER, REJECT_DISTRIB,
+	      REJECT_GROUP, REJECT_UNAPP, REJECT_OTHER} Reject_type;
+
+/*
+**  A newsgroup has a name in different formats, and a high water count,
 **  also kept in different formats.  It also has a list of sites that
 **  get this group.
 */
@@ -434,23 +436,21 @@ typedef enum _FEEDTYPE {
   FTprogram
 } FEEDTYPE;
 
-
 /*
 **  Diablo-style hashed feeds or hashfeeds.
 */
-#define HASHFEED_QH	1
-#define HASHFEED_MD5	2
+#define HASHFEED_QH 1
+#define HASHFEED_MD5 2
 
 typedef struct _HASHFEEDLIST {
-  int			type;
-  unsigned int		begin;
-  unsigned int		end;
-  unsigned int		mod;
-  unsigned int		offset;
-  struct _HASHFEEDLIST	*next;
+  int type;
+  unsigned int begin;
+  unsigned int end;
+  unsigned int mod;
+  unsigned int offset;
+  struct _HASHFEEDLIST *next;
 } HASHFEEDLIST;
 
-
 /*
 **  A site may reject something in its subscription list if it has
 **  too many hops, or a bad distribution.
@@ -499,7 +499,7 @@ typedef struct _SITE {
   struct buffer	  FNLnames;
   int		  Process;
   pid_t		  pid;
-  long		  Flushpoint;
+  size_t	  Flushpoint;
   struct buffer	  Buffer;
   bool		  Buffered;
   char	      **  Originator;
@@ -566,7 +566,7 @@ enum timer {
     TMR_MAX
 };
 
-
+
 
 /*
 **  In-line macros for efficiency.
@@ -584,7 +584,7 @@ enum timer {
     if (--ErrorCount <= 0 || (e) == ENOSPC) \
       ThrottleIOError(WHEN);                \
   } while (0)
-
+
 
 /*
 **  Global data.
@@ -634,7 +634,7 @@ EXTERN sig_atomic_t	GotTerminate;
 EXTERN SITE	    *	Sites;
 EXTERN SITE		ME;
 EXTERN struct timeval	TimeOut;
-EXTERN TIMEINFO		Now;		/* Reasonably accurate time     */
+EXTERN struct timeval	Now;		/* Reasonably accurate time     */
 EXTERN bool		ThrottledbyIOError;
 EXTERN char	    *   NCgreeting;
 EXTERN struct history   *History;
@@ -676,7 +676,6 @@ extern char         *	MaxLength(const char *p, const char *q);
 extern pid_t		Spawn(int niceval, int fd0, int fd1, int fd2,
 			      char * const av[]);
 extern void		CleanupAndExit(int x, const char *why);
-extern void		FileGlue(char *p, const char *n1, char c, const char *n2);
 extern void		JustCleanup(void);
 extern void		ThrottleIOError(const char *when);
 extern void		ThrottleNoMatchError(void);
@@ -694,28 +693,30 @@ extern void		ARTclose(void);
 extern void		ARTsetup(void);
 extern void		ARTprepare(CHANNEL *cp);
 extern void		ARTparse(CHANNEL *cp);
+extern void             ARTlogreject(CHANNEL *cp, const char *text);
+extern void		ARTreject(Reject_type, CHANNEL *);
 
 extern bool		CHANsleeping(CHANNEL *cp);
-extern CHANNEL      *	CHANcreate(int fd, CHANNELTYPE Type,
-				   CHANNELSTATE State,
-				   innd_callback_t Reader,
-				   innd_callback_t WriteDone);
-extern CHANNEL      *	CHANiter(int *cp, CHANNELTYPE Type);
+extern CHANNEL      *	CHANcreate(int fd, enum channel_type Type,
+				   enum channel_state State,
+				   innd_callback_func Reader,
+				   innd_callback_func WriteDone);
+extern CHANNEL      *	CHANiter(int *cp, enum channel_type Type);
 extern CHANNEL      *	CHANfromdescriptor(int fd);
-extern char	    *   CHANname(const CHANNEL *cp);
+extern char	    *   CHANname(CHANNEL *cp);
 extern int		CHANreadtext(CHANNEL *cp);
 extern void		CHANclose(CHANNEL *cp, const char *name);
 extern void		CHANreadloop(void);
 extern void		CHANsetup(int i);
 extern void		CHANshutdown(void);
 extern void		CHANtracing(CHANNEL *cp, bool Flag);
-extern void		CHANsetActiveCnx(CHANNEL *cp);
+extern void		CHANcount_active(CHANNEL *cp);
 
 extern void		RCHANadd(CHANNEL *cp);
 extern void		RCHANremove(CHANNEL *cp);
 
 extern void		SCHANadd(CHANNEL *cp, time_t Waketime, void *Event,
-				 innd_callback_t Waker, void *Argument);
+				 innd_callback_func Waker, void *Argument);
 extern void		SCHANremove(CHANNEL *cp);
 extern void		SCHANwakeup(void *Event);
 
@@ -786,7 +787,7 @@ extern char	    *	RClabelname(CHANNEL *cp);
 extern void		RCclose(void);
 extern void		RChandoff(int fd, HANDOFF h);
 extern void		RCreadlist(void);
-extern void		RCsetup(int i);
+extern void		RCsetup(void);
 
 extern bool		SITEfunnelpatch(void);
 extern bool		SITEsetup(SITE *sp);
@@ -821,21 +822,6 @@ extern bool		WIPinprogress(const char *msgid, CHANNEL *cp,
 extern WIP	    *	WIPbyid(const char *mesageid);
 extern WIP	    *	WIPbyhash(const HASH hash);
 
-/*
-**  TCL globals and functions
-*/
-#if DO_TCL
-extern Tcl_Interp   *	TCLInterpreter;
-extern bool		TCLFilterActive;
-extern struct buffer *	TCLCurrArticle;
-extern ARTDATA	    *	TCLCurrData;
-
-extern void		TCLfilter(bool value);
-extern void		TCLreadfilter(void);
-extern void		TCLsetup(void);
-extern void		TCLclose(void);
-#endif /* DO_TCL */
-
 /*
 **  Python globals and functions
 */
