@@ -1,21 +1,20 @@
-/*  $Revision: 1.2 $
+/*  $Id: sendarticle.c 8016 2008-09-07 13:31:58Z iulius $
 **
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 #include "nntp.h"
 
 
 /*
-**  Send a string of one or more lines down a stdio FILE using RFC977
+**  Send a string of one or more lines down a stdio FILE using RFC 3977
 **  conventions.  Return -1 on error.
 */
-int NNTPsendarticle(char *p, FILE *F, BOOL Terminate)
+int NNTPsendarticle(char *p, FILE *F, bool Terminate)
 {
-    char	        *next;
+    char *next;
 
     for (; p && *p; next[-1] = '\n', p = next) {
 	/* Get pointer to next line.  Truncate long lines. */
