@@ -1,4 +1,4 @@
-/*  $Id: expire.c,v 1.16.2.1 2004/05/17 06:23:42 rra Exp $
+/*  $Id: expire.c 8409 2009-04-11 22:13:41Z iulius $
 **
 **  Code for overview-driven expiration.
 **
@@ -14,18 +14,19 @@
 #include <errno.h>
 
 #include "inn/innconf.h"
-#include "libinn.h"
-#include "ov.h"
+#include "inn/libinn.h"
+#include "inn/ov.h"
 #include "ovinterface.h"
-#include "paths.h"
-#include "storage.h"
+#include "inn/paths.h"
+#include "inn/storage.h"
+#include "inn/vector.h"
 
 enum KRP {Keep, Remove, Poison};
 
 /* Statistics */
-static long             EXPprocessed;
-static long             EXPunlinked;
-static long             EXPoverindexdrop;
+long             EXPprocessed;
+long             EXPunlinked;
+long             EXPoverindexdrop;
 
 #define NGH_HASH(Name, p, j)    \
         for (p = Name, j = 0; *p; ) j = (j << 5) + j + *p++
@@ -92,16 +93,16 @@ static bool             ReadOverviewfmt = false;
 
 /* FIXME: The following variables are shared between this file and ov.c.
    This should be cleaned up with a better internal interface. */
-extern time_t   OVnow;
-extern char *   ACTIVE;
-extern FILE *   EXPunlinkfile;
-extern bool     OVignoreselfexpire;
-extern bool     OVusepost;
-extern bool     OVkeep;
-extern bool     OVearliest;
-extern bool     OVquiet;
-extern int      OVnumpatterns;
-extern char **  OVpatterns;
+time_t   OVnow;
+char *   ACTIVE;
+FILE *   EXPunlinkfile;
+bool     OVignoreselfexpire;
+bool     OVusepost;
+bool     OVkeep;
+bool     OVearliest;
+bool     OVquiet;
+int      OVnumpatterns;
+char **  OVpatterns;
 
 
 /*
@@ -545,56 +546,34 @@ OVEXPremove(TOKEN token, bool deletedgroups, char **xref, int ngroups)
 static void
 ARTreadschema(void)
 {
-    FILE                        *F;
-    char                        *p;
-    char                        *path;
+    const struct cvector        *standardoverview;
+    const struct vector         *extraoverview;
     ARTOVERFIELD                *fp;
-    int                         i;
-    char                        buff[SMBUF];
-    bool                        foundxref = false;
-    bool                        foundxreffull = false;
+    unsigned int                i;
 
-    /* Open file, count lines. */
-    path = concatpath(innconf->pathetc, _PATH_SCHEMA);
-    F = fopen(path, "r");
-    if (F == NULL)
-        return;
-    for (i = 0; fgets(buff, sizeof buff, F) != NULL; i++)
-        continue;
-    fseeko(F, 0, SEEK_SET);
-    ARTfields = xmalloc((i + 1) * sizeof(ARTOVERFIELD));
+    /* Count the number of overview fields and allocate ARTfields. */
+    standardoverview = overview_fields();
+    extraoverview = overview_extra_fields(true);
+    ARTfields = xmalloc((standardoverview->count + extraoverview->count + 1)
+                        * sizeof(ARTOVERFIELD));
 
     /* Parse each field. */
-    for (fp = ARTfields; fgets(buff, sizeof buff, F) != NULL; ) {
-        /* Ignore blank and comment lines. */
-        if ((p = strchr(buff, '\n')) != NULL)
-            *p = '\0';
-        if ((p = strchr(buff, '#')) != NULL)
-            *p = '\0';
-        if (buff[0] == '\0')
-            continue;
-        if ((p = strchr(buff, ':')) != NULL) {
-            *p++ = '\0';
-            fp->NeedsHeader = (strcmp(p, "full") == 0);
-        }
-        else
-            fp->NeedsHeader = false;
+    for (i = 0, fp = ARTfields; i < standardoverview->count; i++) {
+        fp->NeedsHeader = false;
         fp->HasHeader = false;
-        fp->Header = xstrdup(buff);
-        fp->Length = strlen(buff);
-        if (strcasecmp(buff, "Xref") == 0) {
-            foundxref = true;
-            foundxreffull = fp->NeedsHeader;
-        }
+        fp->Header = xstrdup(standardoverview->strings[i]);
+        fp->Length = strlen(standardoverview->strings[i]);
         fp++;
     }
-    ARTfieldsize = fp - ARTfields;
-    fclose(F);
-    if (!foundxref || !foundxreffull) {
-        fprintf(stderr, "'Xref:full' must be included in %s", path);
-        exit(1);
+    for (i = 0; i < extraoverview->count; i++) {
+        fp->NeedsHeader = true;
+        fp->HasHeader = false;
+        fp->Header = xstrdup(extraoverview->strings[i]);
+        fp->Length = strlen(extraoverview->strings[i]);
+        fp++;
     }
-    free(path);
+
+    ARTfieldsize = fp - ARTfields;
 }
 
 /*
@@ -652,7 +631,7 @@ OVERGetHeader(const char *p, int field)
 }
 
 /*
-**  Read overview.fmt and find index for headers
+**  Read overview schema and find index for headers.
 */
 static void
 OVfindheaderindex(void)
@@ -665,7 +644,7 @@ OVfindheaderindex(void)
     if (ReadOverviewfmt)
         return;
     if (innconf->groupbaseexpiry) {
-        ACTIVE = concatpath(innconf->pathdb, _PATH_ACTIVE);
+        ACTIVE = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
         if ((active = ReadInFile(ACTIVE, (struct stat *)NULL)) == NULL) {
             fprintf(stderr, "Can't read %s, %s\n",
             ACTIVE, strerror(errno));
@@ -674,7 +653,7 @@ OVfindheaderindex(void)
         BuildGroups(active);
         arts = xmalloc(nGroups * sizeof(char *));
         krps = xmalloc(nGroups * sizeof(enum KRP));
-        path = concatpath(innconf->pathetc, _PATH_EXPIRECTL);
+        path = concatpath(innconf->pathetc, INN_PATH_EXPIRECTL);
         F = fopen(path, "r");
         free(path);
         if (!EXPreadfile(F)) {
@@ -733,7 +712,8 @@ OVgroupbasedexpire(TOKEN token, const char *group, const char *data,
             EXPoverindexdrop++;
             return true;
         }
-        if ((when = parsedate(p, NULL)) == -1) {
+        when = parsedate_rfc2822_lax(p);
+        if (when == (time_t) -1) {
             EXPoverindexdrop++;
             return true;
         }
@@ -869,9 +849,9 @@ OVEXPcleanup(void)
 
     if (EXPprocessed != 0) {
         if (!OVquiet) {
-            printf("    Article lines processed %8ld\n", EXPprocessed);
-            printf("    Articles dropped        %8ld\n", EXPunlinked);
-            printf("    Overview index dropped  %8ld\n", EXPoverindexdrop);
+            printf("Article lines processed %8ld\n", EXPprocessed);
+            printf("Articles dropped        %8ld\n", EXPunlinked);
+            printf("Overview index dropped  %8ld\n", EXPoverindexdrop);
         }
         EXPprocessed = EXPunlinked = EXPoverindexdrop = 0;
     }
