@@ -1,4 +1,4 @@
-/*  $Id: timehash.c 7412 2005-10-09 03:44:35Z eagle $
+/*  $Id: timehash.c 7960 2008-08-24 09:22:42Z iulius $
 **
 **  Timehash based storage method.
 */
@@ -11,15 +11,15 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <netinet/in.h>
-#include <syslog.h>
 #include <sys/stat.h>
 #include <time.h>
 
 #include "inn/innconf.h"
+#include "inn/messages.h"
 #include "inn/wire.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 #include "methods.h"
-#include "paths.h"
+#include "inn/paths.h"
 #include "timehash.h"
 
 typedef struct {
@@ -58,7 +58,7 @@ static TOKEN MakeToken(time_t now, int seqnum, STORAGECLASS class, TOKEN *oldtok
     return token;
 }
 
-static void BreakToken(TOKEN token, int *now, int *seqnum) {
+static void BreakToken(TOKEN token, time_t *now, int *seqnum) {
     unsigned int        i;
     unsigned short      s = 0;
 
@@ -98,14 +98,14 @@ static TOKEN *PathToToken(char *path) {
 
 bool timehash_init(SMATTRIBUTE *attr) {
     if (attr == NULL) {
-	syslog(L_ERROR, "timehash: attr is NULL");
+        warn("timehash: attr is NULL");
 	SMseterror(SMERR_INTERNAL, "attr is NULL");
 	return false;
     }
     attr->selfexpire = false;
     attr->expensivestat = true;
     if (STORAGE_TOKEN_LENGTH < 6) {
-	syslog(L_FATAL, "timehash: token length is less than 6 bytes");
+        warn("timehash: token length is less than six bytes");
 	SMseterror(SMERR_TOKENSHORT, NULL);
 	return false;
     }
@@ -127,6 +127,8 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
     else
 	now = article.arrived;
 
+    memset(&token, 0, sizeof(token));
+
     for (i = 0; i < 0x10000; i++) {
 	seq = SeqNum;
 	SeqNum = (SeqNum + 1) & 0xffff;
@@ -138,7 +140,7 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
 	    p = strrchr(path, '/');
 	    *p = '\0';
 	    if (!MakeDirectory(path, true)) {
-	        syslog(L_ERROR, "timehash: could not make directory %s %m", path);
+                syswarn("timehash: could not make directory %s", path);
 	        token.type = TOKEN_EMPTY;
 	        free(path);
 	        SMseterror(SMERR_UNDEFINED, NULL);
@@ -147,7 +149,7 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
 	        *p = '/';
 	        if ((fd = open(path, O_CREAT|O_EXCL|O_WRONLY, ARTFILE_MODE)) < 0) {
 		    SMseterror(SMERR_UNDEFINED, NULL);
-		    syslog(L_ERROR, "timehash: could not open %s %m", path);
+                    syswarn("timehash: could not create %s", path);
 		    token.type = TOKEN_EMPTY;
 		    free(path);
 		    return token;
@@ -158,7 +160,8 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
     }
     if (i == 0x10000) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	syslog(L_ERROR, "timehash: all sequence numbers for the time and class are reserved %lu %d", (unsigned long)now, class);
+        warn("timehash: all sequence numbers for time %lu and class %d are"
+             " reserved", (unsigned long) now, class);
 	token.type = TOKEN_EMPTY;
 	free(path);
 	return token;
@@ -167,7 +170,7 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
     result = xwritev(fd, article.iov, article.iovcnt);
     if (result != (ssize_t) article.len) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	syslog(L_ERROR, "timehash error writing %s %m", path);
+        syswarn("timehash: error writing %s", path);
 	close(fd);
 	token.type = TOKEN_EMPTY;
 	unlink(path);
@@ -209,7 +212,7 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
 
     if (fstat(fd, &sb) < 0) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	syslog(L_ERROR, "timehash: could not fstat article: %m");
+        syswarn("timehash: could not fstat article");
 	free(art);
 	return NULL;
     }
@@ -220,7 +223,7 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
     if (innconf->articlemmap) {
 	if ((private->base = mmap(NULL, sb.st_size, PROT_READ, MAP_SHARED, fd, 0)) == MAP_FAILED) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    syslog(L_ERROR, "timehash: could not mmap article: %m");
+            syswarn("timehash: could not mmap article");
 	    free(art->private);
 	    free(art);
 	    return NULL;
@@ -233,7 +236,7 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
 	private->base = xmalloc(private->len);
 	if (read(fd, private->base, private->len) < 0) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    syslog(L_ERROR, "timehash: could not read article: %m");
+            syswarn("timehash: could not read article");
 	    free(private->base);
 	    free(art->private);
 	    free(art);
@@ -270,6 +273,8 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
     if (amount == RETR_HEAD) {
 	art->data = private->base;
 	art->len = p - private->base;
+        /* Headers end just before the first empty line (\r\n). */
+        art->len = art->len - 2;
 	return art;
     }
 
@@ -289,7 +294,7 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
 }
 
 ARTHANDLE *timehash_retrieve(const TOKEN token, const RETRTYPE amount) {
-    int                 now;
+    time_t              now;
     int                 seqnum;
     char                *path;
     ARTHANDLE           *art;
@@ -337,7 +342,7 @@ void timehash_freearticle(ARTHANDLE *article) {
 }
 
 bool timehash_cancel(TOKEN token) {
-    int                 now;
+    time_t              now;
     int                 seqnum;
     char                *path;
     int                 result;
@@ -385,7 +390,9 @@ static struct dirent *FindDir(DIR *dir, FINDTYPE type) {
     return NULL;
 }
 
-ARTHANDLE *timehash_next(const ARTHANDLE *article, const RETRTYPE amount) {
+ARTHANDLE *
+timehash_next(ARTHANDLE *article, const RETRTYPE amount)
+{
     PRIV_TIMEHASH       priv;
     PRIV_TIMEHASH       *newpriv;
     char                *path;
@@ -407,7 +414,7 @@ ARTHANDLE *timehash_next(const ARTHANDLE *article, const RETRTYPE amount) {
     } else {
 	priv = *(PRIV_TIMEHASH *)article->private;
 	free(article->private);
-	free((void *)article);
+	free(article);
 	if (priv.base != NULL) {
 	    if (innconf->articlemmap)
 		munmap(priv.base, priv.len);
@@ -488,7 +495,7 @@ ARTHANDLE *timehash_next(const ARTHANDLE *article, const RETRTYPE amount) {
     newpriv->terde = priv.terde;
     snprintf(path, length, "%s/%s/%s/%s", priv.topde->d_name, priv.secde->d_name, priv.terde->d_name, de->d_name);
     art->token = PathToToken(path);
-    BreakToken(*art->token, (int *)&(art->arrived), &seqnum);
+    BreakToken(*art->token, &(art->arrived), &seqnum);
     free(path);
     return art;
 }
@@ -518,7 +525,8 @@ void
 timehash_printfiles(FILE *file, TOKEN token, char **xref UNUSED,
                     int ngroups UNUSED)
 {
-    int now, seqnum;
+    time_t now;
+    int seqnum;
     char *path;
     
     BreakToken(token, &now, &seqnum);
